/*jslint nomen: true, indent: 2, maxerr: 3 */
/*global window, rJS, Handlebars, document, loopEventListener, RSVP,
   encodeURIComponent */
(function (window, rJS, jIO) {
  "use strict";
  var DEFAULT_APP_LIST_DOC = "app-list.json";

  function getParameterDict() {
    var hash = window.location.hash.substring(1),
      params = {};
    /*
    hash.split('&').map(hk => {
      let temp = hk.split('=');
      params[temp[0]] = temp[1];
    });
    */
    return params;
  }

  function putNewAppList(gadget, app_list) {
    return RSVP.Queue()
      .push(function () {
        return gadget.crib_sw_put(DEFAULT_APP_LIST_DOC, {
          content: JSON.stringify(app_list),
          type: "application/json"
        });
      });
  }

  function getApplicationList(gadget) {
    return RSVP.Queue()
      .push(function () {
        return gadget.crib_sw_get(DEFAULT_APP_LIST_DOC);
      })
      .push(function (data) {
        return jIO.util.readBlobAsText(data, data.type);
      })
      .push(function (data) {
        return JSON.parse(data.target.result);
      });
  }

  function loadCribJSFromZip(gadget, event) {
    var app_list = [],
      redirect_to = gadget.props
        .element.querySelector("form.crib-load-from-zip .load-zip-to-path").value +
        "/" + gadget.props.element.querySelector("form.crib-load-from-zip .redirect-url").value;
    if (gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-file").files.length === 0 &&
        gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-url").value.length === 0) {
      return;
    }
    return RSVP.Queue()
      .push(function () {
        var promise_list = [];
        if (gadget.props.element
              .querySelector("form.crib-load-from-zip .load-zip-file").files.length !== 0) {
          promise_list.push(loadCribJSFromZipFile(gadget, event));
        } else {
          promise_list.push(loadCribJSFromZipUrl(gadget, event));
        }
        promise_list.push(getApplicationList(gadget));
        return RSVP.all(promise_list);
      })
      .push(function (result_list) {
        app_list = result_list[1];
        app_list.splice(
          0, 0, {
            int_index: 41,
            application_image_type: undefined,
            application_image_url: undefined,
            application_description: undefined,
            application_url: redirect_to,
            application_edit_page: redirect_to,
            application_title: gadget.props.element.querySelector(".app-name").value,
            application_category: undefined
          }
        );
        return putNewAppList(gadget, app_list);
      })
      .push(function () {
        document.location = window.location.origin + window.location.pathname + redirect_to;
      });
  }

  function loadCribJSFromZipFile(gadget, event) {
    return RSVP.Queue()
    .push(function () {
      return gadget.getDeclaredGadget("jio_cribjs");
    })
    .push(function (jio_cribjs_gadget) {
      return jio_cribjs_gadget.loadFromZipFile({
        path: document.location.href,
        file_list: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-file").files,
        from_path: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-from-zip-path").value,
        to_path: gadget.props
          .element.querySelector("form.crib-load-from-zip .load-zip-to-path").value,
        application_id: "cribjs"
      });
    });
  }

  function loadCribJSFromZipUrl(gadget, event) {
    return RSVP.Queue()
    .push(function () {
      return gadget.getDeclaredGadget("jio_cribjs");
    })
    .push(function (jio_cribjs_gadget) {
      return jio_cribjs_gadget.loadFromZipUrl({
        path: document.location.href,
        zip_url: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-url").value,
        from_path: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-from-zip-path").value,
        to_path: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-to-path").value,
        application_id: "cribjs"
      });
    });
  }

  rJS(window)
    /////////////////////////////////////////////////////////////////
    // ready
    /////////////////////////////////////////////////////////////////

    // Assign the element to a variable
    .ready(function (g) {
      g.props = {};
      return g.getElement()
        .push(function (element) {
          g.props.element = element;
          g.props.start_deferred = RSVP.defer();
        });
    })
    //////////////////////////////////////////////
    // acquired methods
    //////////////////////////////////////////////
    .declareAcquiredMethod("crib_sw_get", "crib_sw_get")
    .allowPublicAcquisition("crib_sw_get", function (argument_list) {
      return this.crib_sw_get(argument_list[0]);
    })
    .declareAcquiredMethod("crib_sw_put", "crib_sw_put")
    .allowPublicAcquisition("crib_sw_put", function (argument_list) {
      return this.crib_sw_put(argument_list[0], argument_list[1]);
    })
    .declareMethod('render', function (options) {
      var gadget = this,
        getURL = window.location,
        site = getURL.protocol + "//" + getURL.host,
        params = getParameterDict();
      if (params.hasOwnProperty("from_path")) {
        gadget.props.element.querySelector("form.crib-load-from-zip .load-from-zip-path").value = params.from_path;
      }
      if (params.hasOwnProperty("to_path")) {
        gadget.props.element.querySelector("form.crib-load-from-zip .load-zip-to-path").value = params.to_path;
      }
      if (params.hasOwnProperty("zip_url")) {
        gadget.props.element.querySelector("form.crib-load-from-zip .load-zip-url").value = params.zip_url;
      }
      if (params.hasOwnProperty("redirect_url")) {
        gadget.props.element.querySelector("form.crib-load-from-zip .redirect-url").value = params.redirect_url;
      }
      return gadget.props.start_deferred.resolve();
    })
    .declareService(function () {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.props.start_deferred.promise;
        })
        .push(function () {
          var promise_list = [];
          // promise to get content from URL
          promise_list.push(loopEventListener(
            gadget.props.element.querySelector("form.crib-load-from-zip"),
            'submit',
            false,
            function (event) {loadCribJSFromZip(gadget, event); }
          ));
          return RSVP.all(promise_list);
        });
    });



}(window, rJS, jIO));